@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-__version__ = '0.3.2'
+__version__ = '0.3.3'
 
 
 import sys
@@ -58,13 +58,20 @@ class Error (Exception):
 class GeneralError (Error):
 
     def __init__ (self, err):
-        super (GeneralError, self).__init__ ('\n'.join ((line for line in err if line.startswith ('gpg: -:'))), err)
+        super (GeneralError, self).__init__ ('\n'.join ((line.strip () for line in err if line.startswith ('gpg:'))), err)
 
 
 class NoDataError (Error):
+    
+    _reasons = {
+        1: 'No armored data',
+        2: 'Expected a packet but did not found one',
+        3: 'Invalid packet found, this may indicate a non OpenPGP message',
+        4: 'Signature expected but not found'
+    }
 
-    def __init__ (self, err):
-        super (NoDataError, self).__init__ ('No valid data found', err)
+    def __init__ (self, code, err):
+        super (NoDataError, self).__init__ (self._reasons.get (code, 'No valid data found'), err)
 
 
 class UnknownStatusError (Error):
@@ -104,8 +111,9 @@ class InvalidMemberError (Error):
 
     def __init__ (self, err, message, reason = None, who = None):
         if reason is not None:
+            message = message % (reason, who)
             reason = self._reasons [to_int (reason)]
-        super (InvalidMemberError, self).__init__ (message % (reason, who), err)
+        super (InvalidMemberError, self).__init__ (message, err)
 
 
 class SmartcardError (Error):
@@ -169,10 +177,10 @@ class Result (object):
                 raise UnknownStatusError (status, self.err)
 
     def _nodata (self, code, value):
-        raise NoDataError (self.err)
+        raise NoDataError (to_int (value), self.err)
 
     def _sc_op_failure (self, code, value):
-        raise SmartcardError (to_int (value))
+        raise SmartcardError (to_int (value), self.err)
 
     def _inv_member (self, code, value):
         reason, who = value.split ()
@@ -193,7 +201,7 @@ class Result (object):
         raise PassphraseError ('Missing passphrase', self.err)
 
     def _bad_passphrase (self, code, value):
-        raise PassphraseError ('Bad passphrase', self.err)
+        raise PassphraseError ('Bad passphrase for key %s' % value, self.err)
 
     def _decryption_failed (self, code, value):
         raise DecryptionError ('The symmetric decryption failed', self.err)
@@ -359,12 +367,17 @@ class GenKeyResult (Result):
             'KEY_NOT_CREATED': self._key_not_created,
             'KEY_CREATED': self._key_created
         })
+    
+    def handle (self):
+        super (GenKeyResult, self).handle ()
+        if not self.fingerprint:
+            raise GeneralError (self.err)
 
     def _key_not_created (self, code, value):
         raise GeneralError (self.err)
 
     def _key_created (self, code, value):
-        self.fingerprint, self.type = value.split () [:2]
+        self.type, self.fingerprint = value.split () [:2]
 
 
 class SignResult (Result):
@@ -389,6 +402,11 @@ class SignResult (Result):
             'USERID_HINT': self._userid_hint,
             'SIG_CREATED': self._sig_created,
         })
+        
+    def handle (self):
+        super (SignResult, self).handle ()
+        if not self.data:
+            raise GeneralError (self.err)
 
     def _userid_hint (self, code, value):
         _, self.signer = value.split (None, 1)
@@ -463,6 +481,7 @@ class VerifyResult (Result):
         self.fingerprint, _, timestamp, expire_timestamp = value.split (None) [:4]
         self.timestamp = to_timestamp (timestamp)
         self.expire_timestamp = to_timestamp (expire_timestamp)
+        # FIXME: читать поля  <sig-version> <reserved> <pubkey-algo> <hash-algo> <sig-class>
 
     def _errsig (self, code, value):
         self.valid = False
@@ -505,7 +524,8 @@ class EncryptResult (VerifyResult):
 
     def handle (self):
         super (EncryptResult, self).handle ()
-        self.valid = bool (self.data)
+        if not self.data:
+            raise GeneralError (self.err)
 
     def _key_not_created (self, code, value):
         raise GeneralError (self.err)
@@ -520,6 +540,13 @@ class EncryptResult (VerifyResult):
         _, self.signer = value.split (None, 1)
 
 
+class DecryptResult (EncryptResult):
+    
+    def handle (self):
+        super (DecryptResult, self).handle ()
+        self.valid = bool (self.data)
+
+
 class GnuPG (object):
 
     default_key_params = {
@@ -692,7 +719,7 @@ class GnuPG (object):
         '''
         return self.execute (
             ImportResult (),
-            ['--keyserver', keyserver, '--recv-keys'] + make_list (keys)
+            ['--keyserver', keyserver, '--recv-keys'] + list (make_list (keys))
         )
 
     def list_keys (self, secret = False):
@@ -737,7 +764,7 @@ class GnuPG (object):
         '''
         return self.execute (
             DeleteResult (),
-            ['--batch', '--delete-secret-key' if secret else '--delete-key'] + make_list (keys)
+            ['--batch', '--delete-secret-key' if secret else '--delete-key'] + list (make_list (keys))
         )
 
     def key_exists (self, key, secret = False):
@@ -878,7 +905,8 @@ class GnuPG (object):
         if sign_key:
             args += ('--sign', '--default-key', sign_key)
         if always_trust:
-            args.append ('--always-trust')
+            #args.append ('--always-trust')
+            args += ('--trust-model', 'always')
         return self.execute (EncryptResult (), args, passphrase, data_file, True)
 
     def encrypt (self, data, *args, **kwargs):
@@ -918,8 +946,9 @@ class GnuPG (object):
                 os.remove (output_filename)
             args += ('--output', output_filename)
         if always_trust:
-            args.append ('--always-trust')
-        return self.execute (EncryptResult (), args, passphrase, encrypted_file, True)
+            #args.append ('--always-trust')
+            args += ('--trust-model', 'always')
+        return self.execute (DecryptResult (), args, passphrase, encrypted_file, True)
 
     def decrypt (self, encrypted_data, *args, **kwargs):
         '''
